#include <acknex.h>
#include <default.c>

/////////////////////////////////////////////////////////////////////////////////////////////

var cue_angle = 0; // stores the tilt angle for the cue
var balls_left = 0; // stores the number of balls that aren't placed in the pockets yet
var total_score = 0; // stores the score that is displayed on the screen
var cue_dist = 50; // the default distance between the center of the ball and the cue
var restarting = 0; // this variable is set to 1 while the game is being restarted
var max_power = 5000; // play with this value; it gives the maximum shot power

var tempo; // used for debugging

VECTOR pos1; // the 2 vars check if the speed of the ball goes below a certain limit
VECTOR pos2; // and stop it completely if it goes below that limit
VECTOR shot_time = 0; // stores the power of the shot

/////////////////////////////////////////////////////////////////////////////////////////////

STRING* level_wmb = "level.wmb";

/////////////////////////////////////////////////////////////////////////////////////////////

SOUND* ball_wav = "ball.wav"; // ball impact sound
SOUND* scored_wav = "scored.wav"; // ball into pocket sound
SOUND* maxshot_wav = "maxshot.wav"; // maximum shot power was reached

/////////////////////////////////////////////////////////////////////////////////////////////

ENTITY* cue;

ENTITY* myball;

ENTITY* ball1;
ENTITY* ball2;
ENTITY* ball3;
ENTITY* ball4;
ENTITY* ball5;
ENTITY* ball6;
ENTITY* ball7;
ENTITY* ball8;
ENTITY* ball9;

/////////////////////////////////////////////////////////////////////////////////////////////

FONT* arial_font = "Arial#20b";

/////////////////////////////////////////////////////////////////////////////////////////////

PANEL* hud_pan = 
{
	pos_x = 0;
	pos_y = 0;
	layer = 10;
	digits(50, 30,"Balls left: %.f", arial_font, 1, balls_left);
	digits(850, 30, "Total score: %.f", arial_font, 1, total_score);
//	digits(500, 30, "%.f", *, 1000, tempo);
	flags = visible;
}

PANEL* congratulations_pan = // displays the "congratulations" screen at the end of the game
{
	bmap = "congratulations.pcx";
	pos_x = 256;
	pos_y = 250;
	layer = 10;
}

/////////////////////////////////////////////////////////////////////////////////////////////

function move_cue();
function kick_ball();
function restart_game();
function bouncing_sound();

/////////////////////////////////////////////////////////////////////////////////////////////

function main()
{
	fps_max = 75;
	video_mode = 8; // 1024x768 pixels
	video_depth = 32; // 32 bit mode
	video_screen = 1; // start in full screen mode
	level_load (level_wmb);
	wait (3);
	balls_left = 9; // initialize the number of balls

	camera.x = 0; // set the initial position of the camera
	camera.y = 0;
	camera.z = 1600;
	camera.tilt = -90; // as well as its angles (make the camera look down at the ball)
	camera.pan = 90;
	ph_setgravity (vector(0, 0, -386)); // set the gravity

 	myball = ent_create ("pool0.mdl", vector(-500, 0, 50), NULL); // create player's ball
	phent_settype (myball, PH_RIGID, PH_SPHERE); // set the physics entity type
	phent_setmass (myball, 5, PH_SPHERE); // and its mass
	phent_setfriction (myball, 70); // set the friction
	phent_setdamping (myball, 85, 85); // set the damping
	phent_setelasticity (myball, 3, 5); // set the elasticity
	myball.emask |= ENABLE_FRICTION; // the ball is sensitive to collisions
  	myball.event = bouncing_sound; // and runs this function as soon as it collides with something

 	ball1 = ent_create ("pool1.mdl", vector(220, 0, 50), NULL); // create the ball 
	phent_settype (ball1, PH_RIGID, PH_SPHERE); // set the physics entity type
	phent_setmass (ball1, 2, PH_SPHERE); // and its mass
	phent_setfriction (ball1, 60); // set the friction
	phent_setdamping (ball1, 55, 90); // set the damping
	phent_setelasticity (ball1, 10, 30); // set the elasticity
	ball1.emask |= ENABLE_FRICTION; // each ball is sensitive to collisions
  	ball1.event = bouncing_sound; // and uses the same event function

 	ball2 = ent_create ("pool2.mdl", vector(320, -50, 50), NULL); // create the ball 
	phent_settype (ball2, PH_RIGID, PH_SPHERE); // set the physics entity type
	phent_setmass (ball2, 2, PH_SPHERE); // and its mass
	phent_setfriction (ball2, 60); // set the friction
	phent_setdamping (ball2, 55, 90); // set the damping
	phent_setelasticity (ball2, 10, 30); // set the elasticity
	ball2.emask |= ENABLE_FRICTION;
  	ball2.event = bouncing_sound;

 	ball3 = ent_create ("pool3.mdl", vector(320, 50, 50), NULL); // create the ball 
	phent_settype (ball3, PH_RIGID, PH_SPHERE); // set the physics entity type
	phent_setmass (ball3, 2, PH_SPHERE); // and its mass
	phent_setfriction (ball3, 60); // set the friction
	phent_setdamping (ball3, 55, 90); // set the damping
	phent_setelasticity (ball3, 10, 30); // set the elasticity
	ball3.emask |= ENABLE_FRICTION;
  	ball3.event = bouncing_sound;

 	ball4 = ent_create ("pool4.mdl", vector(420, 0, 50), NULL); // create the ball 
	phent_settype (ball4, PH_RIGID, PH_SPHERE); // set the physics entity type
	phent_setmass (ball4, 2, PH_SPHERE); // and its mass
	phent_setfriction (ball4, 60); // set the friction
	phent_setdamping (ball4, 55, 90); // set the damping
	phent_setelasticity (ball4, 10, 30); // set the elasticity
	ball4.emask |= ENABLE_FRICTION;
  	ball4.event = bouncing_sound;

 	ball5 = ent_create ("pool5.mdl", vector(420, -100, 50), NULL); // create the ball 
	phent_settype (ball5, PH_RIGID, PH_SPHERE); // set the physics entity type
	phent_setmass (ball5, 2, PH_SPHERE); // and its mass
	phent_setfriction (ball5, 60); // set the friction
	phent_setdamping (ball5, 55, 90); // set the damping
	phent_setelasticity (ball5, 10, 30); // set the elasticity
	ball5.emask |= ENABLE_FRICTION;
  	ball5.event = bouncing_sound;

 	ball6 = ent_create ("pool6.mdl", vector(420, 100, 50), NULL); // create the ball 
	phent_settype (ball6, PH_RIGID, PH_SPHERE); // set the physics entity type
	phent_setmass (ball6, 2, PH_SPHERE); // and its mass
	phent_setfriction (ball6, 60); // set the friction
	phent_setdamping (ball6, 55, 90); // set the damping
	phent_setelasticity (ball6, 10, 30); // set the elasticity
	ball6.emask |= ENABLE_FRICTION;
  	ball6.event = bouncing_sound;

 	ball7 = ent_create ("pool7.mdl", vector(520, -50, 50), NULL); // create the ball 
	phent_settype (ball7, PH_RIGID, PH_SPHERE); // set the physics entity type
	phent_setmass (ball7, 2, PH_SPHERE); // and its mass
	phent_setfriction (ball7, 60); // set the friction
	phent_setdamping (ball7, 55, 90); // set the damping
	phent_setelasticity (ball7, 10, 30); // set the elasticity
	ball7.emask |= ENABLE_FRICTION;
  	ball7.event = bouncing_sound;

 	ball8 = ent_create ("pool8.mdl", vector(520, 50, 50), NULL); // create the ball 
	phent_settype (ball8, PH_RIGID, PH_SPHERE); // set the physics entity type
	phent_setmass (ball8, 2, PH_SPHERE); // and its mass
	phent_setfriction (ball8, 60); // set the friction
	phent_setdamping (ball8, 55, 90); // set the damping
	phent_setelasticity (ball8, 10, 30); // set the elasticity
	ball8.emask |= ENABLE_FRICTION;
  	ball8.event = bouncing_sound;

 	ball9 = ent_create ("pool9.mdl", vector(620, 0, 50), NULL); // create the ball 
	phent_settype (ball9, PH_RIGID, PH_SPHERE); // set the physics entity type
	phent_setmass (ball9, 2, PH_SPHERE); // and its mass
	phent_setfriction (ball9, 60); // set the friction
	phent_setdamping (ball9, 55, 90); // set the damping
	phent_setelasticity (ball9, 10, 30); // set the elasticity
	ball9.emask |= ENABLE_FRICTION;
  	ball9.event = bouncing_sound;

	cue = ent_create ("cue.wmb", nullvector, move_cue); // create the cue (you can use a better looking wmb / mdl file here)

	wait (3); // wait a bit

	on_mouse_left = kick_ball; // press, hold and then release the left mouse button to hit the ball
	on_r = restart_game; // press the "R" key to restart the game
}

function kick_ball() // hits the ball when the player presses the left mouse button
{
	if (is (cue, INVISIBLE)) {return;} // don't hit the ball if the cue is invisible
	// run this loop until the player releases the mouse button or until the shot power has reached its max
	while ((mouse_left) && (shot_time.x < max_power))
	{
		shot_time.x += 150 * time_step; // increase the power of the shot
		cue_dist += 5 * time_step; // and move the cue backwards
		cue_dist = minv(300, cue_dist); // don't allow cue_dist to exceed 300
		wait (1);
	}
	if (shot_time.x >= max_power) // the maximum power was reached?
	{
		shot_time.x = max_power; // then set the shot_time maximum value
		// and then play this sound; tell the player that it has reached the max (no need to keep the mouse button pressed anymore)
		snd_play(maxshot_wav, 100, 0); 
	}
	while (mouse_left) {wait (1);} // wait until the player releases the left mouse button
	shot_time.y = 0;
	shot_time.z = 0;
	vec_rotate (shot_time, cue.pan); // rotate shot_time in the direction pointed by the cue
	phent_addvelcentral (myball, shot_time); // and add a linear velocity to the ball
	snd_play (ball_wav, 100, 0); // play the ball impact sound
	wait (3); // wait a bit
	shot_time.x = 0; // and then reset the shot power
	cue_dist = 50; // restore the initial cue_dist value
}

function move_cue()
{
	VECTOR temp;
	set(my, PASSABLE); // the cue is made passable
	while (1)
	{
		cue_angle += 3 * (mouse_force.x - mouse_force.y); // and changes its angle according to the movement of the mouse
		my.x = myball.x - cue_dist * sin(cue_angle); // the distance between the cue and the ball is kept at 50 quants at first
		my.y = myball.y + cue_dist * cos(cue_angle);
		my.z = myball.z; // the cue and the ball have the same height; examine cue.wmp in Wed
		vec_set(temp, myball.x); // keep the cue rotated towards the ball at all times
		vec_sub(temp, my.x);
		vec_to_angle(my.pan, temp);
		my.tilt = 0; // but don't alter its tilt
		wait (1);
	}
}

function set_things_startup()
{
	// wait until all the balls and the cue are loaded
	while ((!myball) || (!cue) || (!ball1) || (!ball2) || (!ball3) || (!ball4) || (!ball5) || (!ball6) || (!ball7) || (!ball8) || (!ball9)) {wait (1);}
	while (1)
	{
		while (restarting) {wait (1);} // pause this function while the game is being restarted (the "R" key is pressed)
		vec_set (pos1.x, myball.x); // store the initial position of the ball
		wait (1);
		vec_set (pos2.x, myball.x); // and store it again after a frame
		tempo = vec_dist(pos2.x, pos1.x); // tempo = debugging value
		// if the ball has moved less than 0.5 quants during the last frame (play with this value)
		if (vec_dist (pos2.x, pos1.x) < 0.5) 
		{
			phent_clearvelocity (myball); // stop the ball (it is moving very slow)
			reset (cue, INVISIBLE); // and then make the cue visible, so that we can strike the ball again
		}
		else // the ball is still moving quite fast?
		{
			set (cue, INVISIBLE); // then hide the cue - don't allow the player to strike the ball
		}

		if (myball.z < -150) // player's ball has fallen into a hole?
		{
			set (cue, INVISIBLE); // then hide the cue
			snd_play (scored_wav, 100, 0);
			wait (-1);
			total_score -= 100; // decrease the score if the player puts his own ball into a hole
			total_score = maxv (0, total_score); // but keep a minimum of zero (don't allow negative scores)
			phent_settype (myball, 0, PH_SPHERE); // disable the physics for this ball for a bit
			vec_set (myball.x, vector (-500, 0, 50)); // so that we can put it on the table
			wait (3);
			reset (cue, INVISIBLE); // show the cue again
			phent_settype (myball, PH_RIGID, PH_SPHERE); // register the physics entity again
		}
		
		// one of the balls has fallen into a hole for the very first time?
		if ((ball1.z < -150) && (ball1.z > -370) && (ball1.x > -680) && (ball1.x < 680) && (ball1.y > -370) && (ball1.y < 370) && (ball1.skill1 == 0)) 
		{
			ball1.skill1 = 1;
			snd_play (scored_wav, 100, 0);
			wait (-1);
			total_score += 50; // increase the score
			balls_left -= 1;
		}
		if ((ball2.z < -150) &&  (ball2.z > -370) && (ball2.x > -680) && (ball2.x < 680) && (ball2.y > -370) && (ball2.y < 370) && (ball2.skill1 == 0)) 
		{
			ball2.skill1 = 1;
			snd_play (scored_wav, 100, 0);
			wait (-1);
			total_score += 50; // increase the score
			balls_left -= 1;
		}
		if ((ball3.z < -150) &&  (ball3.z > -370) && (ball3.x > -680) && (ball3.x < 680) && (ball3.y > -370) && (ball3.y < 370) && (ball3.skill1 == 0)) 
		{
			ball3.skill1 = 1;
			snd_play (scored_wav, 100, 0);
			wait (-1);
			total_score += 50; // increase the score
			balls_left -= 1;
		}
		if ((ball4.z < -150) &&  (ball4.z > -370) && (ball4.x > -680) && (ball4.x < 680) && (ball4.y > -370) && (ball4.y < 370) && (ball4.skill1 == 0)) 
		{
			ball4.skill1 = 1;
			snd_play (scored_wav, 100, 0);
			wait (-1);
			total_score += 50; // increase the score
			balls_left -= 1;
		}
		if ((ball5.z < -150) &&  (ball5.z > -370) && (ball5.x > -680) && (ball5.x < 680) && (ball5.y > -370) && (ball5.y < 370) && (ball5.skill1 == 0)) 
		{
			ball5.skill1 = 1;
			snd_play (scored_wav, 100, 0);
			wait (-1);
			total_score += 50; // increase the score
			balls_left -= 1;
		}
		if ((ball6.z < -150) &&  (ball6.z > -370) && (ball6.x > -680) && (ball6.x < 680) && (ball6.y > -370) && (ball6.y < 370) && (ball6.skill1 == 0)) 
		{
			ball6.skill1 = 1;
			snd_play (scored_wav, 100, 0);
			wait (-1);
			total_score += 50; // increase the score
			balls_left -= 1;
		}
		if ((ball7.z < -150) && (ball7.z > -370) && (ball7.x > -680) && (ball7.x < 680) && (ball7.y > -370) && (ball7.y < 370) && (ball7.skill1 == 0)) 
		{
			ball7.skill1 = 1;
			snd_play (scored_wav, 100, 0);
			wait (-1);
			total_score += 50; // increase the score
			balls_left -= 1;
		}
		if ((ball8.z < -150) &&  (ball8.z > -370) && (ball8.x > -680) && (ball8.x < 680) && (ball8.y > -370) && (ball8.y < 370) && (ball8.skill1 == 0)) 
		{
			ball8.skill1 = 1;
			snd_play (scored_wav, 100, 0);
			wait (-1);
			total_score += 50; // increase the score
			balls_left -= 1;
		}
		if ((ball9.z < -150) &&  (ball9.z > -370) && (ball9.x > -680) && (ball9.x < 680) && (ball9.y > -370) && (ball9.y < 370) && (ball9.skill1 == 0)) 
		{
			ball9.skill1 = 1;
			snd_play (scored_wav, 100, 0);
			wait (-1);
			total_score += 50; // increase the score
			balls_left -= 1;
		}
		if (balls_left == 0) // all the balls were placed in the pockets?
		{ 
			break; // then get out of this loop
		}
	}
	set (congratulations_pan, VISIBLE); // the game is over here, so display the "congratulations" panel
	while (key_any) {wait (1);} // wait until the player presses a key
	while (!key_any) {wait (1);}
	sys_exit (NULL); // and then shut down the engine
}

function bouncing_sound() // event function for all the balls
{
	if (you) // collided with another ball? (we don't want to react to collisions with the table)
	{
		snd_play(ball_wav, 70, 0);
  		my.event = NULL; // disable the collisions for 1 second; we do this  because
		wait (-1); // we don't want the ball collision sound to be played too often
  		my.event = bouncing_sound;	
	}
}

function restart_game()
{
	restarting = 1;
	while (key_r) {wait (1);} // wait until the player releases the "R" key
	wait (3);
	phent_settype (myball, 0, PH_SPHERE); // disable the physics for this ball for a bit
	vec_set (myball.x, vector (-500, 0, 50)); // so that we can put it at its initial position on the table
	phent_settype (ball1, 0, PH_SPHERE); // disable the physics for this ball for a bit
	vec_set (ball1.x, vector (220, 0, 50)); // so that we can put it at its initial position on the table
	phent_settype (ball2, 0, PH_SPHERE); // disable the physics for this ball for a bit
	vec_set (ball2.x, vector (320, -50, 50)); // so that we can put it at its initial position on the table
	phent_settype (ball3, 0, PH_SPHERE); // disable the physics for this ball for a bit
	vec_set (ball3.x, vector (320, 50, 50)); // so that we can put it at its initial position on the table
	phent_settype (ball4, 0, PH_SPHERE); // disable the physics for this ball for a bit
	vec_set (ball4.x, vector (420, 0, 50)); // so that we can put it at its initial position on the table
	phent_settype (ball5, 0, PH_SPHERE); // disable the physics for this ball for a bit
	vec_set (ball5.x, vector (420, -100, 50)); // so that we can put it at its initial position on the table
	phent_settype (ball6, 0, PH_SPHERE); // disable the physics for this ball for a bit
	vec_set (ball6.x, vector (420, 100, 50)); // so that we can put it at its initial position on the table
	phent_settype (ball7, 0, PH_SPHERE); // disable the physics for this ball for a bit
	vec_set (ball7.x, vector (520, -50, 50)); // so that we can put it at its initial position on the table
	phent_settype (ball8, 0, PH_SPHERE); // disable the physics for this ball for a bit
	vec_set (ball8.x, vector (520, 50, 50)); // so that we can put it at its initial position on the table
	phent_settype (ball9, 0, PH_SPHERE); // disable the physics for this ball for a bit
	vec_set (ball9.x, vector (620, 0, 50)); // so that we can put it at its initial position on the table

	wait (3);

	balls_left = 9;
	total_score = 0;
	ball1.skill1 = 0;
	ball2.skill1 = 0;
	ball3.skill1 = 0;
	ball4.skill1 = 0;
	ball5.skill1 = 0;
	ball6.skill1 = 0;
	ball7.skill1 = 0;
	ball8.skill1 = 0;
	ball9.skill1 = 0;

	phent_settype (myball, PH_RIGID, PH_SPHERE); // register the physics entity again
	phent_settype (ball1, PH_RIGID, PH_SPHERE); // register the physics entity again
	phent_settype (ball2, PH_RIGID, PH_SPHERE); // register the physics entity again
	phent_settype (ball3, PH_RIGID, PH_SPHERE); // register the physics entity again
	phent_settype (ball4, PH_RIGID, PH_SPHERE); // register the physics entity again
	phent_settype (ball5, PH_RIGID, PH_SPHERE); // register the physics entity again
	phent_settype (ball6, PH_RIGID, PH_SPHERE); // register the physics entity again
	phent_settype (ball7, PH_RIGID, PH_SPHERE); // register the physics entity again
	phent_settype (ball8, PH_RIGID, PH_SPHERE); // register the physics entity again
	phent_settype (ball9, PH_RIGID, PH_SPHERE); // register the physics entity again
	restarting = 0; // the restarting process is over
}
